// SPISample.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <winioctl.h>
#include "SPISample.h"


#define TOPOLY_SPI_ADDRESS 0x0 //SPI4
#define LCD_RESET_GPIO 157
#define BACKLIGHT_GPIO 58
#define SLEEP_TIME_FOR_RESET 75
#define TD035STEE1_LCD_ID    0x2008

BOOL ResetLCD(BOOL leaveLcdInReset = FALSE)
{
	HANDLE hGPIO;
	hGPIO = GPIOOpen();
	if(hGPIO == INVALID_HANDLE_VALUE)
	{
		NKDbgPrintfW(L"Failed to open GPIO handle\r\n");
		return FALSE;
	}
	GPIOSetAsOutput(hGPIO,LCD_RESET_GPIO,FALSE);
	Sleep(SLEEP_TIME_FOR_RESET);
	if(!leaveLcdInReset)
	{
		GPIOSetAsOutput(hGPIO,LCD_RESET_GPIO,TRUE);	
		Sleep(SLEEP_TIME_FOR_RESET);
	}
	GPIOClose(hGPIO);
	return TRUE;
}



BOOL BacklightEnable(BOOL enable)
{
	HANDLE hGPIO;
	hGPIO = GPIOOpen();
	if(hGPIO == INVALID_HANDLE_VALUE)
	{
		NKDbgPrintfW(L"Failed to open GPIO handle\r\n");
		return FALSE;
	}
	GPIOSetAsOutput(hGPIO,BACKLIGHT_GPIO,enable);
	GPIOClose(hGPIO);
	return TRUE;
}




void ConfigureWordLength(HANDLE hSpi, UCHAR length,BOOL isClockActiveHigh)
{
	DWORD config = MCSPI_CH3CONF_REG_BASE | MCSPI_CHCONF_WL(length);
	if(isClockActiveHigh) 
		config |= MCSPI_POL_ACTIVEHIGH;

	if(!SPIConfigure(hSpi, TOPOLY_SPI_ADDRESS,config))
	{
		NKDbgPrintfW(L"Error: Failed to configure SPI4\r\n");
	}
}

void WriteToLCD(HANDLE hSpi, DWORD dwDataToLcd, DWORD *dwDataFromLcd)
{
	if(!SPIWriteRead(hSpi, sizeof(DWORD),&dwDataToLcd,dwDataFromLcd))
	{
		NKDbgPrintfW(L"SPI transaction failed \r\n");	
	}
	Sleep(2);
}

BOOL LCDTurnOn()
{
	NKDbgPrintfW(L"TD035:: LCDTurnOn\r\n");
	DWORD dwDataFromLcd = 0;
	
	int i;
	HANDLE hSpi = SPIOpen(SPI4_DEVICE_NAME);
	if(hSpi == NULL)
	{
		NKDbgPrintfW(L"TD035:: Could not get a handle to SPI4\r\n");
		return FALSE;
	}
	
	if(!ResetLCD() || !BacklightEnable(TRUE))
	{
		goto cleanUp;
	}

	// First 3 commands need to be sent with POL_INACTIVELOW
	ConfigureWordLength(hSpi,9,TRUE);
	

	for (i=0; i < 3 ; ++i)
	{
		//Make a transaction on the bus
		WriteToLCD(hSpi,0x0,&dwDataFromLcd);
	}
	
	
	ConfigureWordLength(hSpi,18,FALSE);
	
	/* send 0x17 to 0xB0 */
	WriteToLCD(hSpi,(0xb0 << 9)+0x117,&dwDataFromLcd);
	/* send 0x80 to 0xBC */
	WriteToLCD(hSpi,(0xbc << 9)+0x180,&dwDataFromLcd);
	/* send 0x00 to 0x3B */	
	WriteToLCD(hSpi,(0x3b << 9)+0x100,&dwDataFromLcd);
	/* send 0x16 to 0xB0 */
	WriteToLCD(hSpi,(0xb0 << 9)+0x116,&dwDataFromLcd);
	
	ConfigureWordLength(hSpi,27,FALSE);
	/* send 0xFF and 0xF9 to 0xB8 */
	WriteToLCD(hSpi,(0xb8 << 18)+(0x1ff << 9)+0x1F9,&dwDataFromLcd);
	ConfigureWordLength(hSpi,9,FALSE);
	/* send 0x11 */
	WriteToLCD(hSpi,0x011,&dwDataFromLcd);
	
	ConfigureWordLength(hSpi,18,FALSE);		
	/* send 0x01 to 0xBA */
	WriteToLCD(hSpi,(0xba << 9)+0x101,&dwDataFromLcd);	
	/* send 0x00 to 0xBB */
	WriteToLCD(hSpi,(0xbb << 9)+0x100,&dwDataFromLcd);	
	/* send 0x50 to 0x3A */
	WriteToLCD(hSpi,(0x3A << 9)+0x150,&dwDataFromLcd);	
	/* send 0x10 to 0xBF */
	WriteToLCD(hSpi,(0xbf << 9)+0x110,&dwDataFromLcd);
	/* send 0x56 to 0xB1 */
	WriteToLCD(hSpi,(0xb1 << 9)+0x156,&dwDataFromLcd);
	/* send 0x33 to 0xB2 */
	WriteToLCD(hSpi,(0xb2 << 9)+0x133,&dwDataFromLcd);
	/* send 0x11 to 0xB3 */
	WriteToLCD(hSpi,(0xb3 << 9)+0x111,&dwDataFromLcd);	
	/* send 0x02 to 0xB4 */
	WriteToLCD(hSpi,(0xb4 << 9)+0x102,&dwDataFromLcd);	
	WriteToLCD(hSpi,(0xb6 << 9)+0x140,&dwDataFromLcd);	
	 /* send 0x03 to 0xB7 */
	WriteToLCD(hSpi,(0xb7 << 9)+0x103,&dwDataFromLcd);	
	/* send 0x04 to 0xB9 */
	WriteToLCD(hSpi,(0xb9 << 9)+0x104,&dwDataFromLcd);
	/* send 0x00 to 0xBE */
	WriteToLCD(hSpi,(0xbe << 9)+0x100,&dwDataFromLcd);
	/* send 0x11 to 0xC0 */
	WriteToLCD(hSpi,(0xc0 << 9)+0x111,&dwDataFromLcd);
	/* send 0x11 to 0xC1 */
	WriteToLCD(hSpi,(0xc1 << 9)+0x111,&dwDataFromLcd);	
	/* send 0x11 to 0xC2 */
	WriteToLCD(hSpi,(0xc2 << 9)+0x111,&dwDataFromLcd);

	ConfigureWordLength(hSpi,27,FALSE);		

	 /* send 0x20 and 0x40 to 0xC3 */
	WriteToLCD(hSpi,(0xc3 << 18)+(0x120 << 9)+0x140,&dwDataFromLcd);
	/* send 0x60 and 0xC0 to 0xC4 */
	WriteToLCD(hSpi,(0xc4 << 18)+(0x160 << 9)+0x1C0,&dwDataFromLcd);	
	/* send 0x10 and 0x20 to 0xC5 */
	WriteToLCD(hSpi,(0xc5 << 18)+(0x110 << 9)+0x120,&dwDataFromLcd);	
	/* send 0x60 and 0xC0 to 0xC6 */
	WriteToLCD(hSpi,(0xc6 << 18)+(0x160 << 9)+0x1C0,&dwDataFromLcd);
	/*send 0x02 and 0x68 to 0xEC */
	WriteToLCD(hSpi,(0xec << 18)+(0x102 << 9)+0x168,&dwDataFromLcd);
	
	ConfigureWordLength(hSpi,18,FALSE);		
	/* send 0x02 to 0xCF */
	WriteToLCD(hSpi,(0xcf << 9)+0x102,&dwDataFromLcd);

	ConfigureWordLength(hSpi,27,FALSE);		
	/*send 0x08 and 0x04 to 0xD0  */
	WriteToLCD(hSpi,(0xd0 << 18)+(0x108 << 9)+0x104,&dwDataFromLcd);
	
	ConfigureWordLength(hSpi,18,FALSE);		
	/* send 0x01 to 0xD1 */
	WriteToLCD(hSpi,(0xd1 << 9)+0x101,&dwDataFromLcd);

	ConfigureWordLength(hSpi,27,FALSE);		
	/* send 0x14 and 0x00 to 0xD2 */
	WriteToLCD(hSpi,(0xd2 << 18)+(0x114 << 9)+0x100,&dwDataFromLcd);
	/* send 0x1A and 0x0F to 0xD3 */
	WriteToLCD(hSpi,(0xd3 << 18)+(0x11a << 9)+0x10f,&dwDataFromLcd);	
	/* send 0x1F and 0xAF to 0xD4 */
	WriteToLCD(hSpi,(0xd4 << 18)+(0x11f << 9)+0x1af,&dwDataFromLcd);
	
	ConfigureWordLength(hSpi,18,FALSE);		
	/* send 0x14 to 0xD5 */
	WriteToLCD(hSpi,(0xd5 << 9)+0x114,&dwDataFromLcd);
	
	ConfigureWordLength(hSpi,9,FALSE);		

	Sleep(40);
	//Send DISPLAY ON command
	WriteToLCD(hSpi,0x029,&dwDataFromLcd);	

	//Read the LCD ID here
	ConfigureWordLength(hSpi,32,FALSE);		
	WriteToLCD(hSpi,0x02000000,&dwDataFromLcd);		

	if ((dwDataFromLcd & 0xFFFF) == TD035STEE1_LCD_ID)
	{
		//After this startup sequence it takes the LCD several seconds to show an image
		RETAILMSG(1,(TEXT("TD035STEE1: Turned display on.\r\n")));
	}
	else
	{
		RETAILMSG(1,(TEXT("TD035STEE1: ERROR. LCD id = 0x%x.\r\n"),(dwDataFromLcd & 0xFFFF)));
		goto cleanUp;
	}
	SPIClose(hSpi);

	NKDbgPrintfW(L"TD035:: Finished LCD VGA init \r\n");
	return TRUE;

cleanUp:
	SPIClose(hSpi);
	return FALSE;
}

BOOL LCDTurnOff()
{
	DWORD dwDataFromLCD;
	NKDbgPrintfW(L"TD035:: LCDTurnOff\r\n");
	HANDLE hSpi = SPIOpen(SPI4_DEVICE_NAME);
	if(hSpi == NULL)
	{
		NKDbgPrintfW(L"TD035:: Could not get a handle to SPI4\r\n");
		return FALSE;
	}

	ConfigureWordLength(hSpi,9,FALSE);
	//Turn Display off
	WriteToLCD(hSpi,0x28,&dwDataFromLCD);
	ResetLCD(TRUE);
	BacklightEnable(FALSE);
	CloseHandle(hSpi);
	return TRUE;
}



int _tmain(int argc, _TCHAR* argv[])
{
	LCDTurnOff();
	Sleep(1000);
	LCDTurnOn();
	return 0;
}



