//
//  File:  SPISample.h
//
#ifndef __SPISAMPLE_H
#define __SPISAMPLE_H

#ifdef __cplusplus
extern "C" {
#endif


//------------------------------------------------------------------------------
//
//  Define:  GPIO_DEVICE_NAME
//
#define GPIO_DEVICE_NAME        L"GIO1:"

//------------------------------------------------------------------------------
//
//  Define:  GPIO_DIR_xxx/GPIO_INT_xxx
//
#define GPIO_DIR_OUTPUT         (0 << 0)
#define GPIO_DIR_INPUT          (1 << 0)






#define IOCTL_GPIO_SETBIT       \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0300, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_CLRBIT       \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0301, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_GETBIT       \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0303, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_SETMODE      \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0304, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_GETMODE      \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0305, METHOD_BUFFERED, FILE_ANY_ACCESS)




__inline HANDLE GPIOOpen()
{
    HANDLE hDevice = INVALID_HANDLE_VALUE;

    hDevice = CreateFile(GPIO_DEVICE_NAME, 0, 0, NULL, 0, 0, NULL);
    if (hDevice == INVALID_HANDLE_VALUE) 
	    {
        DEBUGMSG(1, (L"GPIOOpen CreateFile failed\r\n"));
	    return NULL;
        }

    return hDevice;
}

__inline void GPIOClose(HANDLE hGPIO)
{
    if (hGPIO != NULL)
    {
        CloseHandle(hGPIO);
    }
}



__inline void GPIOSetAsOutput(HANDLE hGPIO,DWORD num, BOOL isOutputHigh)
{
	DWORD   dwValue[2];

    dwValue[0] = num;
    dwValue[1] = GPIO_DIR_OUTPUT;

    DeviceIoControl(hGPIO,
					IOCTL_GPIO_SETMODE,
					&dwValue,
					sizeof(dwValue),
					NULL,
					0,
					NULL,
					NULL );

	if(isOutputHigh)
		DeviceIoControl(hGPIO,
                        IOCTL_GPIO_SETBIT,
                        &num,
                        sizeof(num),
                        NULL,
                        0,
                        NULL,
                        NULL );
	else
		DeviceIoControl(hGPIO,
                        IOCTL_GPIO_CLRBIT,
                        &num,
                        sizeof(num),
                        NULL,
                        0,
                        NULL,
                        NULL );	
}


__inline void GPIOSetAsInput(HANDLE hGPIO,DWORD num)
{
	DWORD   dwValue[2];
	
	dwValue[0] = num;
    dwValue[1] = GPIO_DIR_INPUT;

    DeviceIoControl(hGPIO,
					IOCTL_GPIO_SETMODE,
					&dwValue,
					sizeof(dwValue),
					NULL,
					0,
					NULL,
					NULL );
}


__inline BOOL IsGpioOutput(HANDLE hGPIO,DWORD num)
{
	DWORD mode = 0;

    DeviceIoControl(hGPIO,
                    IOCTL_GPIO_GETMODE,
                    &num,
                    sizeof(num),
                    &mode,
                    sizeof(mode),
                    NULL,
                    NULL );

	return (!(mode & GPIO_DIR_INPUT));
}

__inline BOOL IsGpioInput(HANDLE hGPIO, DWORD num)
{	
	return (!IsGpioOutput(hGPIO,num));
}

__inline BOOL IsGpioInputHigh(HANDLE hGPIO, DWORD num)
{
	DWORD   dwValue = 0;

	DeviceIoControl(hGPIO,
                    IOCTL_GPIO_GETBIT,
                    &num,
                    sizeof(num),
                    &dwValue,
                    sizeof(dwValue),
                    NULL,
                    NULL );

     return dwValue;
}

__inline BOOL IsGpioInputLow(HANDLE hGPIO, DWORD num)
{
	return (!IsGpioInputHigh(hGPIO,num));
}




#define SPI4_DEVICE_NAME         L"SPI4:"

#define IOCTL_SPI_CONFIGURE     \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0200, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_SPI_WRITEREAD     \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0201, METHOD_BUFFERED, FILE_ANY_ACCESS)


/* CHCONF Register Bit values */
#define MCSPI_PHA_ODD_EDGES                     (0 << 0)
#define MCSPI_PHA_EVEN_EDGES                    (1 << 0)

#define MCSPI_POL_ACTIVEHIGH                    (0 << 1)
#define MCSPI_POL_ACTIVELOW                     (1 << 1)

#define MCSPI_CHCONF_CLKD(x)                    ((x & 0x0F) << 2)

#define MCSPI_CSPOLARITY_ACTIVEHIGH             (0 << 6)
#define MCSPI_CSPOLARITY_ACTIVELOW              (1 << 6)

#define MCSPI_CHCONF_WL(x)                      (((x-1) & 0x1F) << 7)
#define MCSPI_CHCONF_GET_WL(x)                  (((x >> 7) & 0x1F) +1)

#define MCSPI_CHCONF_TRM_TXRX                   (0 << 12)
#define MCSPI_CHCONF_TRM_RXONLY                 (1 << 12)
#define MCSPI_CHCONF_TRM_TXONLY                 (2 << 12)

#define MCSPI_CHCONF_DMAW_ENABLE                (1 << 14)
#define MCSPI_CHCONF_DMAW_DISABLE               (0 << 14)

#define MCSPI_CHCONF_DMAR_ENABLE                (1 << 15)
#define MCSPI_CHCONF_DMAR_DISABLE               (0 << 15)

#define MCSPI_CHCONF_DPE0                       (1 << 16)
#define MCSPI_CHCONF_DPE1                       (1 << 17)
#define MCSPI_CHCONF_IS                         (1 << 18)
#define MCSPI_CHCONF_TURBO                      (1 << 19)
#define MCSPI_CHCONF_FORCE                      (1 << 20)


#define MCSPI_CH3CONF_REG_BASE 		MCSPI_PHA_EVEN_EDGES | \
									MCSPI_POL_ACTIVELOW | \
									MCSPI_CHCONF_CLKD(8) | \
									MCSPI_CSPOLARITY_ACTIVELOW | \
									MCSPI_CHCONF_TRM_TXRX | \
									MCSPI_CHCONF_DMAW_DISABLE | \
									MCSPI_CHCONF_DMAR_DISABLE| \
									MCSPI_CHCONF_DPE0



typedef struct {
    DWORD address;
    DWORD config;
} IOCTL_SPI_CONFIGURE_IN;


__inline HANDLE SPIOpen(LPCTSTR pSpiName)
{
    HANDLE hDevice;


    hDevice = CreateFile(pSpiName, 0, 0, NULL, 0, 0, NULL);
	if (hDevice == INVALID_HANDLE_VALUE) {
		return NULL;
	}
    return hDevice;
}

__inline VOID SPIClose(HANDLE hDevice)
{
    CloseHandle(hDevice);
}


__inline BOOL SPIConfigure(HANDLE hDevice, DWORD address, DWORD config)
{
    IOCTL_SPI_CONFIGURE_IN  dwIn;

    dwIn.address = address;
    dwIn.config = config;
    
    return DeviceIoControl(hDevice, 
                    IOCTL_SPI_CONFIGURE, 
                    &dwIn,
                    sizeof(dwIn),
                    NULL,
                    0,
                    NULL,
                    NULL );
    
}

__inline DWORD SPIWriteRead(HANDLE hDevice, DWORD size, VOID *pOutBuffer, VOID *pInBuffer)
{

    return DeviceIoControl(hDevice, 
                    IOCTL_SPI_WRITEREAD, 
                    pInBuffer,
                    size,
                    pOutBuffer,
                    size,
                    NULL,
                    NULL );

}

#ifdef __cplusplus
}
#endif

#endif